/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gesture for Mozilla.
 *
 * The Initial Developer of the Original Code is Pavol Vaskovic.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Andy Edmonds <aedmonds@mindspring.com>
 *  David Illsley <illsleydc@bigfoot.com>
 *  HJ van Rantwijk <bugs4HJ@netscape.net>
 *  Jens Bannmann <jens.b@web.de>
 *  Martin.T.Kutschker <Martin.T.Kutschker@blackbox.net>
 *  Pavol Vaskovic <pali@pali.sk>
 *  Pekka Aleksi Knuutila <aleksi.knuutila@edu.lahti.fi>
 *  Scott R. Turner <srt@aero.org>
 *  Tim Williamson <chsman@hotmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// Global variables
// mouse gesture condition (to be evaluated on event) for more details see mozgestReadPref()
var condition = new String;
var grid; // grid size; minimal gesture has to be 'grid' pixels long
var delay; // delay before aborting gesture
var lmbGestureLimit;
var gestureStartTime;
var dTolerance; // tolerance for diagonal movement. see processCoordinates()
var gestureInProgress = false;
// old coordinates from previous gesture stroke used in processCoordinates()
var old_x;
var old_y;
var gridMoves = new Array();
var gesture = new Array();
var localizedGesture = new Array(); var R,L,D,U,D1,D3,D7,D9;
var lastgesture;
// lastGestureTime is time of last gesture stroke, compared with delay and current time
var lastGestureTime;
var lastMoveTime;
// timer used to abort gestures after delay, see processCoordinates()
var timer;

// trail settings
var drawTrail;
var trailLimit;
var trailWidth;
var trailInterval;
var trailColor;
var trailOpacity;

// globalOnLink is an array that holds a list of all the links traversed during gesture
// globalOnImage is a string containing an image href or false
// globalSrcEvent stores event which started the gesture the active gesture.
// The links are used in the gesture functions in gesture.js.
var globalOnLink = false;
var globalOnImage = false;
var globalSrcEvent = false;
var inContent = false;
var bundle = null; // String bundle for localized strings
var disableContextMenu = false;
var mouseButtonsPressed = new Array();

// Preferences observer object; implements nsIObserver
function mgPrefObserver()
{
  try {
    var pbi = pref.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver(this.domain, this, false);
  } catch(ex) {
    dump("MozGest: Failed to observe prefs: " + ex + "\n");
  }
}
mgPrefObserver.prototype =
{
  domain: "mozgest.",
  observe: function(subject, topic, prefName)
  {
    // when MozGest pref was changed, we reinitialize
    try{
      mozgestInit();
    } catch(err){}
  }
}

// This code is performed on load:
// get the preferences service
var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);
var pref = prefService.getBranch(null); // preferences root node
var mgPref = prefService.getBranch("mozgest."); // preferences mozgest node
window.mozgestPrefObserver = new mgPrefObserver(); // and set an Pref observer
// END of on load performed code

function mozgestDisableContextMenu(aEvent) {
  if (disableContextMenu) {
    aEvent.preventDefault();
  }
  disableContextMenu = false;
}

function mozgestInit() { // load preferences into JS variables, or restore default prefs if none are found
  dump("MozGest: init\n");
  window.removeEventListener("load", mozgestInit, true);

  // read prefs (or set default for each missing setting)
  mozgestReadPref();

  // load localized string bundle & init dependand Gesture Table
  bundle = document.getElementById("mozgestbundle");
  R = bundle.getString("abbreviation.right");
  L = bundle.getString("abbreviation.left");
  U = bundle.getString("abbreviation.up");
  D = bundle.getString("abbreviation.down");
  D1 = bundle.getString("abbreviation.d1");
  D3 = bundle.getString("abbreviation.d3");
  D7 = bundle.getString("abbreviation.d7");
  D9 = bundle.getString("abbreviation.d9");
  initGestureTable();

  // context menu disabler for right mouse button
  var contentAreaContextMenu = mgGetContextMenu();
  if (contentAreaContextMenu)
    contentAreaContextMenu.addEventListener("popupshowing", mozgestDisableContextMenu, false);
}

function processCoordinates(e){ // e is browser's event object
  var g = gesture.join("");
  if (condition == "e.button==0" && (g == "L" || g == "R")
      && (e.timeStamp - gestureStartTime) > lmbGestureLimit) {
    endGesture(null);
  }

  mozgestDrawTrail(e);

  var x_dir = e.clientX - old_x;
  var y_dir = e.clientY - old_y;
  var x = Math.abs(x_dir);
  var y = Math.abs(y_dir);

  if ( (x >= grid/2 || y >= grid/2) ) { // process each half-grid
    // diagonal movement:
    // dTolerance = 75 means that a movement is recognized as diagonal when 
    // x/y or y/x is between 0.25 and 1
    if ( (x/y >= (1-dTolerance/100) && x/y <= 1)
        || (y/x >= (1-dTolerance/100) && y/x <= 1) ) {

      if (x_dir < 0 && y_dir > 0) {
        gPush("1", D1);
      } else if (x_dir > 0 && y_dir > 0) {
        gPush("3", D3);
      } else if (x_dir < 0 && y_dir < 0) {
        gPush("7", D7);
      } else if (x_dir > 0 && y_dir < 0) {
        gPush("9", D9);
      }

    }
    // horizontal movement:
    else if (x > y) {
      if (x_dir > 0) {
        gPush("R", R);
      } else if (x_dir < 0) {
        gPush("L", L);
      }
    }
    // vertical movement:
    else if (x < y) {
      if (y_dir > 0){
        gPush("D", D);
      } else if (y_dir < 0){
        gPush("U", U);
      }
    }
    mgMsg(bundle.getString("g.gesture") + " " + localizedGesture);
    lastGestureTime = e.timeStamp;
    if (timer) { window.clearTimeout(timer); }
    timer = window.setTimeout("endGesture(null);", delay);

    old_x = e.clientX;
    old_y = e.clientY;
  }
  lastMoveTime = e.timeStamp;

  mozgestHandleLink(e);
}

function gPush(code, localCode) {
  gridMoves.push(new Array(code, localCode));

  // Rules: 1) a diagonal must be in both half-grids to be recognized
  //        2) non-diagonal moves have precedence

  if (gridMoves.length == 2) {
    // Second half-grid is diagonal
    if (isNaN(gridMoves[1][0]) == false) {
      // add it if first half-grid is the same direction
      if (gridMoves[1][0] == gridMoves[0][0]) {
        doPush(gridMoves[1][0], gridMoves[1][1]);
      }
      // if first half-grid is non-diagonal, add this instead of the diagonal
      else if (isNaN(gridMoves[0][0])) {
        doPush(gridMoves[0][0], gridMoves[0][1]);
      }
    }
    // Second half-grid is non-diagonal: add it
    else {
      doPush(gridMoves[1][0], gridMoves[1][1]);
    }
    gridMoves.length = 0;
  }
}

function doPush(code, localCode) {
  if (gesture[gesture.length-1] != code) {
    gesture.push(code);
    localizedGesture.push(localCode);
  }
}

function mozgestHandleLink(e){ // add new traversed link to globalOnLink array
  var aHref = findLink(e.target);
  if(aHref) {
    if(!globalOnLink) globalOnLink = new Array();
    if(!mozgestCheckLinkDups(aHref)){
      dump("Mozgest: adding new link to globalOnLink\n");
      globalOnLink.push(aHref);
   }
  }
}

// Recruited this function to find images, AE 11/15/01
// It loops up the DOM looking for a link and returns the A node
function findLink(domPosition) { // traverses DOM, searches for links
  //dump("walking dom" + domPosition.nodeName + " \n");
  if (domPosition == window._content.document) return false;
  try { // This is needed when user drags outside of content area,
        // because then we run out of tree up to null - 1st 'if' is never true.
     if(domPosition.nodeName == "IMG" && globalOnImage == false){
      globalOnImage = domPosition;
          dump("Image found " + domPosition.src + "  \n");
     }
    if(domPosition.nodeName == "A") return domPosition.href;
    else return findLink(domPosition.parentNode);
  }
  catch(e) {return false;}
}

function mozgestCheckLinkDups(addy) { // avoid duplicates in globalOnLink array
  var isADup = false;
  for(var zz = 0; zz < globalOnLink.length; zz++) {
    if(globalOnLink[zz] == addy) isADup = true;
  }
  return isADup;
}

// startGesture is called from event listener in bubble phase,
// it relays on mozgestIsInContent which is called in capture phase
// to determine if gesture was started in the right place
function startGesture(e){
  mouseButtonsPressed[e.button] = true;
  if (inContent){
    if(e.button == "0" && mouseButtonsPressed["2"]){
      //fire "back" event
      disableContextMenu = true;
      fireGesture("L");
      
      gestureInProgress = false;
      gestureStartTime = false;
      globalOnLink = false;
      globalOnImage = false;
      globalSrcEvent = false;
      window.removeEventListener("mousemove", processCoordinates, false);
      mouseButtonsPressed["0"] = false; mouseButtonsPressed["2"] = false; 
    }
    else if(e.button == "2" && mouseButtonsPressed["0"]){
      //fire forward event
      disableContextMenu = true;
      fireGesture("R");
      
      gestureInProgress = false;
      gestureStartTime = false;
      globalOnLink = false;
      globalOnImage = false;
      globalSrcEvent = false;
      window.removeEventListener("mousemove", processCoordinates, false);
      mouseButtonsPressed["0"] = false; mouseButtonsPressed["2"] = false; 
    }
    else if (eval(condition)){ // is condition (mouse button, modifiers) for gesture met?
      globalSrcEvent = e;
      // DO NOT start gesture on scrollbars and input elements etc.
      var tag = e.target.nodeName.toLowerCase();
      // textarea is a bit tricky... #text is in 'div' which is in 'textarea'
      // when there's no error, we end with textarea or something harmless
      if (tag == "#text") {
        try { tag = e.target.parentNode.parentNode.nodeName.toLowerCase(); }
        catch (err) {}
      }
      var xtag;
      try { xtag = e.originalTarget.localName.toLowerCase(); } catch (err) {}

      if (xtag != "slider" && xtag != "thumb" && xtag != "scrollbarbutton" &&
          tag != "input" && tag != "select" && tag != "textarea" &&
          tag != "textbox") {
        gestureInProgress = true;
        gestureStartTime = e.timeStamp;
        lastMoveTime = 0;
        old_x = e.clientX;
        old_y = e.clientY;
        mozgestHandleLink(e);  // I could have started over link, let's check!
        gesture.length = 0; // clear gesture array
        localizedGesture.length = 0; // clear localized gesture array
        mozgestBeginTrail(e);
        window.addEventListener("mousemove", processCoordinates, false);
      }
    }
    lastGestureTime = e.timeStamp;
    inContent = false;
  }
}

function endGesture(e){
  if(e) mouseButtonsPressed[e.button] = false;
  if (gestureInProgress) {
    window.removeEventListener("mousemove", processCoordinates, false);
    mozgestEndTrail();
    lastgesture = gesture.join("");

    // check if we've been called via a timeout; see processCoordinates()
    if (e == null) {
      mgMsg(bundle.getString("g.aborted"));
    } else if (lastgesture != ""){
      /* we should stop default actions here (popup menus, text selecting)
       but I think that current Mozilla implementation does those things
       internaly, so we can not stop them from DOM. */
      e.preventDefault();
      e.stopPropagation();

      // Disable context menu but only if the right mouse button
      // is configured as the trigger
      if(mgPref.getIntPref("mousebutton") == 2) disableContextMenu = true;

      fireGesture(lastgesture);
    } 
    else dump("MozGest: Gesture was too small\n");

    gestureInProgress = false;
    gestureStartTime = false;
    globalOnLink = false;
    globalOnImage = false;
    globalSrcEvent = false;
  }
}

function mozgestReadPref(){ // read settings from Preferences interface
  grid = mgGetPref("grid", "Int", 15, 10, 500);
  delay = mgGetPref("delay", "Int", 500);
  lmbGestureLimit = mgGetPref("lmbGestureLimit", "Int", 500, 100, 10000);
  dTolerance = mgGetPref("diagonalTolerance", "Int", 60, 0, 75);

  // condition is a boolean expression consisting of event object values
  condition = "e.button==" + mgGetPref("mousebutton", "Int", 0, 0, 2);
  if (mgGetPref("modifier.ctrl", "Bool", false))
    condition += " && e.ctrlKey";
  if (mgGetPref("modifier.alt", "Bool", false))
    condition += " && e.altKey";
  if (mgGetPref("modifier.shift", "Bool", false))
    condition += " && e.shiftKey";

  drawTrail = mgGetPref("trails.enabled", "Bool", false);
  trailWidth = mgGetPref("trails.width", "Int", 5, 1, 100);
  trailInterval = mgGetPref("trails.interval", "Int", 2, 1, 100);
  trailColor = mgGetPref("trails.color", "Char", "#FF0000");
  trailLimit = mgGetPref("trails.length-limit", "Int", 1000, 1, 10000);
  trailOpacity = mgGetPref("trails.opacity", "Int", 100, 1, 100);

  // turn gestures recognition on/off
  if (mgIsEnabled()) {
    addWindowWatch();
  } else {
    removeWindowWatch();
  }
}

// Gets a pref setting. If it's missing, set it to a default value.
// For Int prefs, you can optionally specify a minimum and maximum value.
function mgGetPref(name, type, defaultValue, min, max) {
  if (type != "Bool" && type != "Char" && type != "Int") {
    return null;
  }
  var notfound = false;
  var value;
  try {
    value = eval("mgPref.get" + type + "Pref(\"" + name + "\");");
  } catch (e) {
    notfound = true;
  }
  if (notfound || (type == "Int" && (min || max) && (value > max || value < min))) {
    val = (type == "Char") ? ("\"" + defaultValue + "\"") : defaultValue;
    eval("mgPref.set" + type + "Pref(\"" + name + "\", " + val + ");");
    return defaultValue;
  } else {
    return value;
  }
}

function addWindowWatch() {
  window.addEventListener("mousedown", startGesture, false);
  window.addEventListener("mouseup", endGesture, false);
  dump("MozGest: ---- Adding Window Listener ---\n");
}

function removeWindowWatch() {
  window.removeEventListener("mousedown", startGesture, false);
  window.removeEventListener("mouseup", endGesture, false);
  dump("MozGest: ---- Removing Listener ---\n");
}
